#include "Riostream.h"
#include "TFile.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"

using namespace std;

void histoPeak(){
    
    //open the root file to read
    TFile* file = TFile::Open("./data_B0toKpi.root");
    //and take the tree with the method Get()
    TTree* tree = (TTree*) file->Get("dataTree");
    
    //just a trivial check
    int tot_entries = tree->GetEntries();
    cout << "Total entries in the tree: " << tot_entries << endl;
    
    //define the variable we want to access to
    double B_m, B_de;

    //and link them to the branch address of the tree
    tree->SetBranchAddress("B_m",&B_m);
    tree->SetBranchAddress("B_de",&B_de);
    
    //just two histograms to fill
    TH1D* h_m_sig = new TH1D("h_m_sig",
							 ";m(B) [GeV/c^{2}]; Entries",
							 30,5.26,5.29);
	h_m_sig->Sumw2();// very important!
  
	//let's clone the histogram for the background
    TH1D* h_m_bkg = (TH1D*) h_m_sig->Clone("h_m_bkg");
  
    //loop over the entries
    for(int iEntry; iEntry<tot_entries; ++iEntry){
        
		//take an entry
        tree->GetEntry(iEntry);
        
		//fill the histograms
        if(fabs(B_de)>0.1) h_m_bkg->Fill(B_m);
        else if(fabs(B_de)<0.05) h_m_sig->Fill(B_m);
	    
    }
  
	//let's clone the histogram
	TH1D* h_m_peak = (TH1D*) h_m_sig->Clone("h_m_peak");
	//Let's subtract the histogram of the background!
	h_m_peak->Add(h_m_bkg,-1);
  
    //draw the histograms
    TCanvas* c1 = new TCanvas("c1","c1",1200,500);
    c1->Divide(2,1);
	c1->cd(1);
    h_m_sig->SetLineColor(kBlue);
	h_m_sig->SetMarkerColor(kBlue);
    h_m_sig->SetLineWidth(2);
    h_m_sig->Draw();

    h_m_bkg->SetLineColor(kRed);
	h_m_bkg->SetMarkerColor(kRed);
	h_m_bkg->SetMarkerStyle(24);
	h_m_bkg->Draw("same");
    
	c1->cd(2);
	h_m_peak->Draw();
    
  
    return;
}
