#include "Riostream.h"
#include "TString.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TTree.h"
#include "TFile.h"
#include "TLorentzVector.h"

using namespace std;

void makeTree(){

	TH1D* h_px = new TH1D("h_K_px", " ; p_{x}(K) [GeV/c]; ",40,-3.2,3.2);
    h_px->GetYaxis()->SetTitle(Form("Candidates per %.1f [Gev/c]",
                                   h_px->GetXaxis()->GetBinWidth(1)));

	//histograms of the new variables
	TH1D* h_B_de = new TH1D("h_B_de", " ; #DeltaE [GeV]; counts", 20, -0.15, 0.15);
	TH1D* h_B_m = new TH1D("h_B_m", " ; M(B) [GeV/c^{2}]; counts", 20, 5.26, 5.29);
  
	//a 2D histogam of the new variables
	TH2D* h_B_de_m = new TH2D("h_B_de_m", " ; #DeltaE [GeV]; M(B) [GeV/c^{2}]", //titles...
							  30,-0.15, 0.15, //binning and range in x-axis
							  20, 5.26, 5.29); //binning and range in y-axis
  
  
    string file_name ="data_file.txt";
    ifstream file_in(file_name);
    
    if(!file_in.is_open()) {
        cout << "Cannot open data file:" << file_name << endl;
        return;
    }
    
    int icand = 0;
    double k_px,  k_py,  k_pz;
    double pi_px, pi_py, pi_pz;

	//define the new variables
	double B_m, B_de;
	//and those needed for the calculation
	const double pi_m = 0.1396;
	const double k_m = 0.4937;
	const double sqrt_s = 10.5794;
    
    //Will store all variables in a format
    //called a TTree, a root dataformat
    //very conveniet to aggregate data
    //in several dimensions
    //https://root.cern.ch/doc/master/classTTree.html
    TTree* dataTree = new TTree("dataTree","B0toKpi data");
    
    //the K momentum components
    dataTree->Branch("k_px",&k_px,"k_px/D");
    dataTree->Branch("k_py",&k_py,"k_py/D");
    dataTree->Branch("k_pz",&k_pz,"k_pz/D");
    //the pi momentum components
    dataTree->Branch("pi_px",&pi_px,"pi_px/D");
    dataTree->Branch("pi_py",&pi_py,"pi_py/D");
    dataTree->Branch("pi_pz",&pi_pz,"pi_pz/D");
	//add the new variables to the tree
	dataTree->Branch("B_m",&B_m,"B_m/D");
	dataTree->Branch("B_de",&B_de,"B_de/D");
 
    while(file_in.is_open()){

        file_in >> k_px  >> k_py  >> k_pz
                >> pi_px >> pi_py >> pi_pz;
               
        if(file_in.eof()) break;
                       
        h_px->Fill(k_px);
	  
		//define the 4-momenta of the kaon and pion
		TLorentzVector k_p, pi_p;
		k_p.SetXYZM(k_px,k_py,k_pz,k_m);
		pi_p.SetXYZM(pi_px,pi_py,pi_pz,pi_m);
	
		//Compute the B 4-momentum
		TLorentzVector B_p = k_p + pi_p;
	  
	    //take the B energy with B_p.E(), and calcuate DeltaE
		B_de = B_p.E() - sqrt_s/2;
		//take the B momentum vector with B_p.Vect()
	    //calculate the magnitude squared with .Mag2()
	    //and compute the mass
		B_m = sqrt( sqrt_s*sqrt_s/4 - B_p.Vect().Mag2() );
	  
        //fill the tree
        dataTree->Fill();
        
		//fill the new histograms
		h_B_de->Fill(B_de);
		h_B_m->Fill(B_m);
		h_B_de_m->Fill(B_de,B_m);
	  
        ++icand;
    }
    
    file_in.close();
    
	cout << "Number of candidates: " << icand << endl;
    
    //make a trivial check...
    cout << "Candidates in the tree: " << dataTree->GetEntries() << endl;
    //look at the content
    dataTree->Print();
  
	TCanvas* canv = new TCanvas("canv","canv",1200,1000);
	canv->Divide(2,2); //split the canvas in 2x2 parts
	
	canv->cd(1);//enter the first part
	//a few drawing options
	h_B_m->SetMarkerStyle(4);
	h_B_m->SetMarkerSize(1);
	h_B_m->SetMarkerColor(kRed+3);
	h_B_m->SetLineColor(kRed+3);
	h_B_m->SetMinimum(0);
	h_B_m->Draw("err");//draw with error bars
  
	canv->cd(2);//enter the second part
	h_B_de_m->Draw("COLZ");
  
	canv->cd(4);//enter the fourth part
	h_B_de->SetFillStyle(3001);
	h_B_de->SetFillColor(kBlue-4);
    h_B_de->SetLineColor(kBlue);
    h_B_de->SetLineWidth(2);
	h_B_de->SetMinimum(0);
	h_B_de->Draw("histo");
  
	canv->cd(3); //enter the third part
	h_px->Draw();
  
  
    //store now in a root file
    TFile* dataFile = new TFile("data_B0toKpi.root","RECREATE");
    dataTree->Write();
    h_px->Write();
    dataFile->Close();
      
    return;

}
